/**
 * Copyright Qnext Inc. 2002-2020
 * http://www.qnext.com
 * Date: 29/12/20
 * Time: 18:49
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 * Frame that is used to show currently and set new Connector list mode.
 * See FF-7814, FF-8109, FF-8275.
 *
 * @param {jQuery} $container
 * @param {Object} options
 * @param {Boolean} doNotRequestStatus
 * @constructor
 */
var FrameListModeSettings = function ($container, options, doNotRequestStatus) {
    FrameWithForm.call(this, FrameListModeSettings.VID, $container, options, doNotRequestStatus);
};

QnUtil.inherits(FrameListModeSettings, FrameWithForm);
QnUtil.extend(FrameListModeSettings, {

    /* Public method */

    updateFrame: function () {
        // Hide "Cancel" buttons for browsers
        this._toggleBtn(BTN_CANCEL_ID);

        // Use scheme for list mode - see FF-8109
        // Then revert and use list-mode - see FF-8275
        var listMode = getGlobal(KEY_GLOBAL_LIST_MODE_CUR);
        var listModeDisable = getGlobal(KEY_GLOBAL_LIST_MODE_DISABLE);
        switch (listMode) {
            case LIST_MODE_PC:
            case LIST_MODE_SERVER:
                // We support only pc or server.
                log("Got list mode: '" + listMode + "' disabled=" + listModeDisable);
                break;
            default:
                // In first 2 calls to updateFrame we have no status response yes and
                // scheme is invalid here
                if (!listMode)
                    return;
                logWarn("Unknown list mode: '" + listMode + "'");
                listModeDisable = true;
                listMode = null;
                break;
        }
        var $comp = this._getCont().find(".listModeChoice");

        if (listMode) {
            // select default
            var found = false;
            $('input[name=list_mode]').each(function () {
                if ($(this).val() === listMode) {
                    $(this).prop("checked", true);
                    found = true;
                    logDbg("Select listMode=" + listMode);
                }
            });
            if (!found)
                logWarn("Cannot find listMode=" + listMode);
        }
        if (!listModeDisable) {

            // set tooltip desc text
            setTextContext("list_mode_settings_info");
            // add focus handler
            $comp.on("focus", "input", function () {
                switchTT($comp.parent());
            });
            // add hover handler
            $comp.hover(function () {
                switchTT($comp.parent());
            }, function () {
                $("body").find(".tooltipCont").css("display", "none");
                $(':focus').focus();
            });

            FrameListModeSettings.superclass.updateFrame.apply(this);

            this._updateSubmitBtnState();
        } else {
            // Disable all components if NO list mode to be used
            $('input[name=list_mode]').each(function () {
                $(this).prop("disabled", true);
            });
            this._getBtn(BTN_SAVE_ID).prop("disabled", true);
        }
    },


    /* Protected method */

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getBtnIdList: function () {
        return [BTN_SAVE_ID, BTN_CANCEL_ID];
    },

    /**
     *
     * @return {string} always valid can be empty
     * @protected
     */
    _getControlIdPrefix: function () {
        return "connector-";
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderPageTitle: function ($cont) {
        var $desc = $('<h2 class="secureonly"></h2>')
            .setTextById("list_mode_settings_title");
        $desc.appendTo($cont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderFormImpl: function ($container) {
        this._renderListModeChoice($container);
        // bind radio button listener
        var self = this;
        $container.on("change", $(':radio'),function (e) {
            self._handleGuiEvent($(this), e);
        });
        // render tooltip
        this._addTooltip($container, "ListMode", "list_mode_settings");
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitImpl: function (event) {
        var hasChanged = this._hasChanges();
        // confirmation dlg on save of list mode - see https://jira2.cnexus.com/browse/FF-8516
        hasChanged = confirm(QnLocale.getStr("list_mode_settings_save_confirm"));
        return hasChanged;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
        if (resp.result) {
            var redirectURL = resp["redirectURL"];
            log("Connector list mode settings update successful");
            // indicate success visually
            showMessage(ERROR_OK, "list_mode_settings_success", this._getMainCont().find("#updatemsgForm"));

            // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
            if (redirectURL != null) {
                // if there is a redirect URL, we immediately redirect the user
                log("redirectURL found: " + redirectURL);
                window.location.href = redirectURL;
            } else {
                log("no redirect URL was found");
            }

        } else {
            log("Connector list mode settings update failure");
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
        if (xhr.responseText) {
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                log("Connector list mode settings update failed error code: " + error["code"]);
            } catch (e) {
                log("Connector list mode settings update failed: " + e);
            }
        }
    },

    /**
     * Enable/Disable form controls (inputs, buttons etc..)
     *
     * @param {Boolean} [onDisable]
     * @param {Boolean} [notForSubmit]
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        // update input
        this._getMainCont().find("input").attr("disabled", onDisable);
        // update buttons
        var $btnSubmit = this._getBtn(BTN_SAVE_ID);
        if(!notForSubmit)
            $btnSubmit.button(onDisable ? 'loading' : 'reset');
        $btnSubmit.attr("disabled", onDisable);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        // We have no separated command like "set-list-mode"
        // but we use "set-global" with parameter "list-mode". See FF-8275
        params["cmd"] = "set-global";
        params[KEY_GLOBAL_LIST_MODE_CUR] = $('input[name=list_mode]:checked').val();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getRequestUrl: function () {
        return "/secure/";
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleGuiEvent: function ($comp, event) {
        this._updateSubmitBtnState();
    },


    /* Private method */

    /**
     * Render list mode choice
     *
     * @param {jQuery} $cont
     * @private
     */
    _renderListModeChoice: function ($cont) {
        var $compCont = this.createCont().addClass("listModeChoice").appendTo($cont);
        var $comp = $('<div class="form-group"></div>');
        var radioPrefix = "list_mode_";
        for (var i = 0; i < FrameListModeSettings._LIST_MODE.length; i++) {
            var mode = FrameListModeSettings._LIST_MODE[i];
            var id = radioPrefix + mode;
            var $radioButton = $('<input type="radio" name="list_mode" id="' + id +
                '" class="form-control field_list_mode_input" value="' + mode + '">')
                .appendTo($comp);
            $('<span class="field_list_mode_label"></span>')
                .setTextById("field_" + id).appendTo($comp);
            $('<br/>').appendTo($comp);

        }
        $comp.appendTo($compCont);
    },

    /**
     * Enable/Disable save button
     *
     * @private
     */
    _updateSubmitBtnState: function () {
        this._getBtn(BTN_SAVE_ID).prop("disabled", !this._hasChanges());
    },

    /**
     * Check did GUI is changed from default values that was set on init
     *
     * @return {Boolean} always valid
     * @private
     */
    _hasChanges: function () {
        var hasChange = false;
        var listMode = getGlobal(KEY_GLOBAL_LIST_MODE_CUR);
        // check proxy configuration type
        var currCompMode = $('input[name=list_mode]:checked').val();
        if (currCompMode !== listMode)
            hasChange = true;

        return hasChange;
    }

});

/**
 *
 * @type {string}
 * @constant
 */
FrameListModeSettings.VID = "FrameListModeSettings";

/**
 * List with available list mode
 *
 * @type {String[]}
 * @constant
 */
FrameListModeSettings._LIST_MODE = [
    LIST_MODE_PC,
    LIST_MODE_SERVER
];
