/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 8.3.2018
 * Time: 15:28
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {String} type of proxy configuration one of {@link ProxyConfig#TYPE_HTTP} ...
 * @param {Boolean} isDetected flag determine did proxy configuration is {@link ProxyConfig#CONFIG_TYPE_AUTO}
 * @param {String} [host]
 * @param {Number} [port]
 * @param {String} [user]
 * @param {String} [pass]
 * @constructor
 */
var ProxyConfig = function (type, isDetected, host, port, user, pass) {
    if(!QnUtil.isDefined(type))
        throw new Error("Invalid Proxy type: " + type);
    if(!QnUtil.isBool(isDetected))
        throw new Error("Not boolean Proxy param isDetected: " + isDetected);
    if(QnUtil.isDefined(port)) {
        if(!QnUtil.isNumber(port) || (port < 1 && port > 65535))
            throw new Error("Invalid Proxy port: " + port);
    }

    /**
     *
     * @type {String} one of {@link ProxyConfig#CONFIG_TYPE_NONE} ...
     * @private
     */
    this._configType = null;

    switch (type) {
        case ProxyConfig.TYPE_NOT_SET:
            this._configType = isDetected ? ProxyConfig.CONFIG_TYPE_AUTO : ProxyConfig.CONFIG_TYPE_NONE;
            break;
        case ProxyConfig.TYPE_HTTP:
            if(!port)
                port = 3127;
            this._configType = ProxyConfig.CONFIG_TYPE_MANUAL;
            break;
        case ProxyConfig.TYPE_SOCKS_4:
        case ProxyConfig.TYPE_SOCKS_5:
            if(!port)
                port = 1337;
            this._configType = ProxyConfig.CONFIG_TYPE_MANUAL;
            break;
        default:
            throw new Error("Unknown proxy type=" + type);
    }

    if(this._configType === ProxyConfig.CONFIG_TYPE_MANUAL && type === ProxyConfig.TYPE_NOT_SET) {
        throw new Error("Invalid proxy configuration: type=" + type + ", isDetected=" + isDetected +
            ", _configType=" + this._configType);
    }

    /**
     *
     * @type {String}
     * @private
     */
    this._type = type;
    /**
     *
     * @type {Boolean}
     * @private
     */
    this._isDetected = isDetected;
    /**
     *
     * @type {String}
     * @private
     */
    this._host = host || "";
    /**
     *
     * @type {Number}
     * @private
     */
    this._port = port ? port : NaN;
    /**
     *
     * @type {String}
     * @private
     */
    this._user = user || "";
    /**
     *
     * @type {String}
     * @private
     */
    this._pass = pass || "";

};

QnUtil.extend(ProxyConfig.prototype, {

    /* Public method */

    /**
     * Get configuration type
     *
     * @return {String} always valid ont of {@link ProxyConfig#CONFIG_TYPE_NONE} ...
     */
    getConfigType: function () {
        return this._configType;
    },

    /**
     *
     * @return {Boolean} always valid
     */
    isConfigManual: function () {
        return this._configType === ProxyConfig.CONFIG_TYPE_MANUAL;
    },

    /**
     *
     * @return {String} always valid
     */
    getType: function () {
        return this._type;
    },

    /**
     *
     * @return {String} can be empty string
     */
    getHost: function () {
        return this._host;
    },

    /**
     *
     * @return {Number} can be NaN
     */
    getPort: function () {
        return this._port;
    },

    /**
     *
     * @return {String} can be empty string
     */
    getUsername: function () {
        return this._user;
    },

    /**
     *
     * @return {String} can be empty string
     */
    getPassword: function () {
        return this._pass;
    },

    /**
     *
     * @param {object} obj
     */
    toProto: function (obj) {
        obj["type"] = this._type;
        obj["detect"] = this._isDetected;
        obj["server"] = this._host;
        obj["port"] = isNaN(this._port) ? "" : this._port;
        obj["user"] = this._user;
        obj["password"] = this._pass;
    }


    /* Protected method */
    /* Private method */

});

/**
 * No proxy configuration. Will not use proxy
 *
 * @type {string}
 * @constant
 */
ProxyConfig.CONFIG_TYPE_NONE = "None";
/**
 *  Auto detected proxy configuration (use system proxy configuration)
 *
 * @type {string}
 * @constant
 */
ProxyConfig.CONFIG_TYPE_AUTO = "Auto";
/**
 * Manually specify proxy configuration
 *
 * @type {string}
 * @constant
 */
ProxyConfig.CONFIG_TYPE_MANUAL = "Manual";
/**
 *
 * @type {string}
 * @constant
 */
ProxyConfig.TYPE_NOT_SET = "NOTSET";
/**
 *
 * @type {string}
 * @constant
 */
ProxyConfig.TYPE_HTTP = "HTTP";
/**
 *
 * @type {string}
 * @constant
 */
ProxyConfig.TYPE_SOCKS_4 = "SOCKS4";
/**
 *
 * @type {string}
 * @constant
 */
ProxyConfig.TYPE_SOCKS_5 = "SOCKS5";


/**
 *
 * @type {ProxyConfig}
 */
ProxyConfig.NOT_SET = new ProxyConfig(ProxyConfig.TYPE_NOT_SET, false);

/**
 * Parse proxy from status info
 *
 * @param {Object} root raw proxy object
 * @return {ProxyConfig}
 */
ProxyConfig.fromStatus = function (root) {
    var type = root["type"];
    var isDetected = root["detect"];
    var host = root["server"];
    var port = parseInt(root["port"]);
    var user = root["user"];

    return new ProxyConfig(type, isDetected, host, port, user);
};
