/**
 * Copyright Qnext Inc. 2002-2019
 * http://www.qnext.com
 * Date: 14.01.2019
 * Time: 12:00
 * Author Milen Dimitrov (milend@qnext.com)
 */

/**
 *
 * Main object for localization. See FF-3098.
 */
var QnLocale = {};

/**
 * List with entries in same format as JSON.
 * E.g.:
 *  <?xml version="1.0" encoding="UTF-8"?>
 *  <xliff xmlns="urn:oasis:names:tc:xliff:document:1.1">
 *      <file target-language="en" source-language="en">
 *          <body>
 *                <trans-unit id="text_encrypt_header_2"      // String UID. Obligatory.
 *                    xml:space="preserve"
 *                    weblate-flags="read-only,safe-html"     // use to determine format. Optional, default=plain.
 *                                                            // If safe-html, format=html otherwise plain.
 *              >
 *                    <source>text_encrypt_header_2</source>
 *                    <target>Intel</target>                  // String text/value. Obligatory.
 *                </trans-unit>
 *  ....
 *
 * @type {object} Always valid
 * @private
 */
QnLocale._listEntry = {};

/**
 * Default locale.
 *
 * @type {string}
 * @private
 * @constant
 */
QnLocale._LOCALE_DEF = "en";

/**
 * List with Locales which are allowed.
 * French added in FF-11153.
 *
 * @type {string[]}
 * @private
 * @constant
 */
QnLocale._LIST_LOCALE = ["en", "ja", "es", "it", "fr"]; // No "bg" because not up to date

/**
 * Format of string is HTML, so html() method should be used.
 * @type {string}
 * @constant
 */
QnLocale.FORMAT_HTML = "html";
/**
 * Format of string is Plain text, so text() method should be used.
 * @type {string}
 * @constant
 */
QnLocale.FORMAT_PLAIN = "plain";

/**
 * @typedef {Object} StrEntry
 * @property {string} value the text of the string
 * @property {string} [format]. Possible values: "html", "text".
 */


/**
 * Initialize localization. Must be called for each page!
 *
 * @param {Function} [callback] to be called when loading finished. Must be valid.
 * @param {boolean} [applyAttr=false] If true, calls {@link QnLocale#setTextFromAttr}
 * when strings are loaded.
 */
QnLocale.initLocale = function (callback, applyAttr) {
    var locale = getGlobal(KEY_GLOBAL_LOCALE);
    // Fix/normalize locale because it can be invalid or not-allowed
    var localeFix = QnLocale._fixLocale(locale);
    if (localeFix !== locale) {
        // noinspection JSUnresolvedVariable
        _LIST_GLOBALS[KEY_GLOBAL_LOCALE] = localeFix;
        locale = localeFix;
    }

    if (applyAttr) {
        var callbackOrg = callback;
        callback = function () {
            try {
                QnLocale.setTextFromAttr();
            } catch (e) {
                logWarn("Cannot apply default texts", e);
            }
            if (callbackOrg)
                callbackOrg();
        }
    }

    // First load English/Default so we can be sure that we always have a fallback!
    QnLocale._loadRes("admin", QnLocale._LOCALE_DEF, function (succeeded) {
        if (succeeded && locale !== QnLocale._LOCALE_DEF) {
            QnLocale._loadRes("admin", locale, function (/*succeeded*/) {
                callback();
            });
        } else {
            callback();
        }
    });
};

/**
 * Returns localized string for given string uid.
 * There maybe added additional arguments. See {@link #formatMessage}.
 *
 * @param {string} uidStr string uid to be localized
 * @param {Object} [opts] optional options. See below.
 * @param {Object} [opts.listArg] list with arguments. Optional. See QnUtil.formatMessage for format
 * @param {boolean} [opts.noEsc=false] to NOT escape passed arguments (by default they are escaped)
 * @param {boolean} [opts.isSpec=false] true to try to find first string with UID "[uidStr]_[OS]_[SCHEME]".
 * @param {boolean} [opts.isOpt=false] to NOT escape passed arguments (by default they are escaped)
 * @return {string} can be null ONLY if isOpt=true and no such string
 */
QnLocale.getStr = function (uidStr, opts) {
    var entry = QnLocale._getEntryInt(uidStr, opts);
    var str = QnLocale._formatEntry(uidStr, entry, opts);
    return str;
};

/**
 * Checks is there a string with given ID.
 *
 * @param {String} uidStr string uid to be localized
 * @return {boolean} if there is such string (can be in current locale or in the default one).
 */
QnLocale.hasStr = function (uidStr) {
    var entry = QnLocale._getEntry(uidStr, false);
    return QnUtil.isDefined(entry);
};

/**
 * Format message with given arguments. Argument in strIn should be specified
 * with %NameArg% and escape %% with %. For example:
 * formatMessage("Address %ip% was banned. Loading 99%%.", {ip:"192.168.0.1"}) will
 * produce: "Address 192.168.0.1 was banned. Loading 99%."
 * @param {String} strIn string to be formatted
 * @param {Object} [listArg] list with arguments. If undefined or empty original string
 * will be returned.
 * @param {Boolean} [noEsc=false] to NOT escape passed arguments (by default they are escaped)
 */
QnLocale.formatMessage = function (strIn, listArg, noEsc) {
//    if (.isInfo())
//        log(QnQbg.toStringObject("Format: " + strIn + "'", listArg));

    if (!strIn) {
        logWarn("string to format");
        return strIn;
    }


    // NOTE we always have to go through the format function as even if there are no specifically passed
    // arguments (e.g. listArg is null or empty) are there are some default arguments
    // that can be existing in the string like %appName%

    // pass strOut inside a reference so it can be updated in the recursion function QnLocale._format(...)
    var result = {
        strOut: ""
    };
    QnLocale._format(strIn, listArg, noEsc, result, 0);

    //QNLOG.info("Format '" + strIn + "' -> '" + result.strOut + "'");
    return result.strOut;
};

/**
 * Iterate recursively all children components of the given container
 * which has attribute "data-i18n", get its value and call
 * {@link #setTextById} with this value.
 * This is used for backward compatibility with previous localization system.
 *
 * @param {jQuery} [$cont] root container to iterate recursively all children.
 * Can be invalid, then HTML document is assumed.
 */
QnLocale.setTextFromAttr = function ($cont) {
    if (!$cont)
        $cont = $(document);
    // Milen: Not good, but structure above is too complex to repeat
    $cont.find("[data-i18n]").each(function () {
        var $cur = $(this);
        $cur.setTextById($cur.attr("data-i18n"));
    });
};

/**
 * Check input locale for valid/allowed value.
 * If it is in format "bg_BG" country code will be removed and only language will be used.
 * Moreover the list with allowed locales will be checked and if input is not allowed,
 * the default will be returned
 *
 * @param {string} localeIn any value.
 * @return {string} fixed locale. Always valid, can be same as input. Can be the default one.
 * @private
 */
QnLocale._fixLocale = function (localeIn) {
    var locale = localeIn;
    if (!locale) {
        locale = navigator.language;
        if (locale) {
            log("Use locale from browser: " + locale);
        }
    } else {
        log("Use locale from param: " + locale);
    }
    if (locale) {
        // Do not use country code
        var pos = locale.indexOf("-");
        if (pos > 0)
            locale = locale.substring(0, pos);
        if (QnLocale._LIST_LOCALE.indexOf(locale) < 0) {
            log("Not allowed locale: " + locale);
            locale = QnLocale._LOCALE_DEF;
        }
    } else {
        locale = QnLocale._LOCALE_DEF;
        log("Use default locale: " + locale);
    }
    return locale;
};

/**
 * Load a single bundle (JSON file) for concrete locale.
 *
 * @param {String} module module name. For example "admin".
 * @param {String} locale locale ID. For example: "en" or "bg".
 * @param {Function} callback to be called when loading finished. Must be valid.
 * @private
 */
QnLocale._loadRes = function (module, locale, callback) {
    // If under /secure/ - make 2 levels up. Not so nice, but we dont want to
    var path = window.location.pathname.indexOf("/secure/") >= 0 ? "../../" : "";
    path += "locales/" + module + "-" + locale + ".xlf";

    $.ajax({
        method: "GET",
        url: path,
        dataType: "xml"
    }).done(function (resp) {
        log("Locale loaded : " + path);

        try {
            var $xmlDoc = $(resp);
            var $entries = $xmlDoc.find("trans-unit");
            if (!$entries || !QnUtil.isObject($entries) || $entries.length <= 0) {
                // noinspection ExceptionCaughtLocallyJS
                throw new Error("Missing entries");
            }
            var isDef = locale === QnLocale._LOCALE_DEF;
            if (isDef) {
                QnLocale._listEntry = {};
            }
            var toLog = isLogDbg();
            var list = QnLocale._listEntry;
            $.each($entries, function (index, entry) {
                var $entry = $(entry);
                var key = $entry.attr("id");
                var value = $entry.find("target").text();
                if (QnUtil.isStr(key) && QnUtil.isStr(value)) {
                    if (isDef) {
                        // Default is plain (plaintext)
                        var format;
                        var attrFormat = $entry.attr("datatype");
                        if (attrFormat) {
                            switch (attrFormat) {
                                case "plaintext":
                                    format = QnLocale.FORMAT_PLAIN;
                                    break;
                                case "html":
                                    format = QnLocale.FORMAT_HTML;
                                    break;
                                default:
                                    logWarn("Unknown format " + attrFormat + " for " + key);
                                    format = QnLocale.FORMAT_PLAIN;
                                    break;
                            }
                        }
                        // This is the default, so set whole entry
                        var clone = {
                            "value": value
                        };
                        if (format !== QnLocale.FORMAT_PLAIN)
                            clone["format"] = format;
                        list[key] = clone;
                        if (toLog)
                            logDbg("Def string: " + key + "='" + value + "', format=" + format);
                    } else {
                        var entryDef = list[key];
                        if (entryDef) {
                            // This is not default, Override only value
                            entryDef["value"] = value;
                            if (toLog)
                                logDbg("Trans string: " + key + "='" + value + "'");
                        } else {
                            logWarn("Not default entry for " + key + " in locale " + locale);
                        }
                    }
                } else {
                    logWarn("Invalid entry: " + key + "=" + value);
                }
            });

            callback(true);
        } catch (e) {
            logWarn("Cannot load locale " + locale, e);
            callback(false);
        }
    }).fail(function (jqXHR, textStatus, ex) {
        logWarn("Locale loading failed: " + path + jqXHR.status + " " + textStatus +
            (jqXHR.responseText ? ", response: " + jqXHR.responseText : "") +
            (ex ? ", exception: " + ex : ""), ex);
        // TODO Milen: What to do if failed?
        callback(false);
    });
};

/**
 * Creates a device specific string UID in format:
 * [uidStrIn]_[OS]_[SCHEME]
 *
 * @param {string} uidStr string UID
 * @return {string} always valid
 * @private
 */
QnLocale._uidStrDevice = function (uidStr) {
    var os = getGlobal(KEY_GLOBAL_DEVICE_OS);
    if (os)
        uidStr += "_" + os;
    var scheme = getGlobal(KEY_GLOBAL_DEVICE_SCHEME);
    if (scheme)
        uidStr += "_" + scheme;
    return uidStr;
};

/**
 * Returns localized string entry for given string uid.
 *
 * @param {String} uidStr string uid to be localized
 * @param {Boolean} realGet this is for getStr, not for hasStr.
 *  If true - trace warning if string is not found.
 * @return {StrEntry|null} always valid
 * @private
 */
QnLocale._getEntry = function (uidStr, realGet) {
    if (!uidStr)
        throw new Error("uidStr");
    var le = QnLocale._listEntry;
    var entry = le[uidStr];
    if (QnUtil.isDefined(entry)) {
        if (isLogDbg())
            logDbg("Found string: " + uidStr + "='" + entry + "'");
        return entry;
    } else {
        if (realGet)
            logWarn("No string: '" + uidStr + "'", new Error("Stack"));
        return null;
    }
};

/**
 * Get string entry specific for device.
 * If isSpec=true, it will try to get string with uid created by:
 * [uidStr]_[OS]_[SCHEME]
 * where OS and SCHEME are device OS and Scheme (get from global parameters).
 * If no such string or isSpec=false, get simply uidStr.
 *
 * @param {String} uidStr string uid.
 * @param {Object} opts optional options. See below.
 * @param {boolean} [opts.noEsc=false] to NOT escape passed arguments (by default they are escaped)
 * @param {boolean} [opts.isSpec=false] true to try to find first string with UID "[uidStr]_[OS]_[SCHEME]".
 * @param {boolean} [opts.isOpt=false] to NOT escape passed arguments (by default they are escaped)
 * @return {StrEntry} can be null if no entry even in default UID
 */
QnLocale._getEntryInt = function (uidStr, opts) {
    var entry;
    if (opts && opts["isSpec"]) {
        var uidSpec = QnLocale._uidStrDevice(uidStr);
        entry = QnLocale._getEntry(uidSpec, false);
    }
    if (!entry)
        entry = QnLocale._getEntry(uidStr, true);
    return entry;
};

/**
 *
 * @param {String} uidStr string uid.
 * @param {StrEntry} entry entry to be formatted. Can be null.
 * @param {Object} opts optional options. See below.
 * @param {boolean} [opts.noEsc=false] to NOT escape passed arguments (by default they are escaped)
 * @param {boolean} [opts.isSpec=false] true to try to find first string with UID "[uidStr]_[OS]_[SCHEME]".
 * @param {boolean} [opts.isOpt=false] to NOT escape passed arguments (by default they are escaped)
 * @return {string} can be null if no entry even in default UID and isOpt=true.
 * @private
 */
QnLocale._formatEntry = function (uidStr, entry, opts) {
    var listArg = opts ? opts["listArg"] : null;
    var noEsc = opts && opts["noEsc"];
    var str = entry ? QnLocale.formatMessage(entry.value, listArg, noEsc) :
        (opts && opts["isOpt"] ? null : "?" + uidStr);
    return str;
};

/**
 * Note this is recursive function.
 * @param {string} strIn input string to be formatted.
 * @param {object} [listArg] list with arguments. If undefined or empty original string
 * @param {boolean} [noEsc=false] to NOT escape passed arguments (by default they are escaped)
 * @param {object} result output result to be fill in.
 * @param {number} startIndex index into original string to start from.
 *  Used when call in recursion.
 * @private
 */
QnLocale._format = function (strIn, listArg, noEsc, result, startIndex) {

    // Index(number) of first found % if have any
    var currIndex = strIn.indexOf("%", startIndex);
    // Check did have any %. If not copy string like it is
    if (currIndex >= 0) {
        // Get index of next %
        var nextIndex = strIn.indexOf("%", currIndex + 1);
        // After found one % check did next symbol is %
        if (currIndex + 1 === nextIndex) {
            nextIndex = strIn.indexOf("%", nextIndex + 1);
            // Check did have any more %. If not nextIndex will be end of input string
            if (nextIndex < 0)
                nextIndex = strIn.length;
            // String between two last found % and next one or if not have any more to end of input string
            var tmp = strIn.substring(startIndex, nextIndex);
            // Next symbol was % and we make escape (%% = %) and concatenate output string with escaped one
            result.strOut += tmp.replace("%%", "%");
        } else {
            // Concatenate output string with part of input string.
            // This part is from last place that was found % to the next found one
            result.strOut += strIn.substring(startIndex, currIndex);
            var change = "";
            // If not have any more % we cant have fragments.
            // In this case just select rest of string and put in in result.strOut
            if (nextIndex < 0) {
                nextIndex = strIn.length;
                change = strIn.substring(currIndex, nextIndex);
            } else {
                // This is part that need to be replaced with one from listArgs.
                var fragment = strIn.substring(currIndex, nextIndex + 1);
                // Remove % from begin and end of fragment
                var key = fragment.substring(1, fragment.length - 1);
                if (key === "appName") {
                    var appName = window["_QN_APP_NAME"] || "FileFlex";
                    change = "" + appName;
                    // Escape listArg[key]. If noEsc is true will not escape it.
                    if (true !== noEsc)
                        change = change.escapeHtml();
                } else if (listArg && Object.prototype.hasOwnProperty.call(listArg, key)) {
                    // Changed fragment with key found in listArgs
                    change = "" + listArg[key];
                    // Escape listArg[key]. If noEsc is true will not escape it.
                    if (true !== noEsc)
                        change = change.escapeHtml();
                } else {
                    change = fragment;
                    logWarn("No key found in listArgs for fragment: " + fragment);
                }

                var lastIndex = nextIndex;
                nextIndex = strIn.indexOf("%", nextIndex + 1);
                // Check did have any more %. If not nextIndex will be end of input string
                if (nextIndex < 0)
                    nextIndex = strIn.length;

                change += strIn.substring(lastIndex + 1, nextIndex);
            }
            result.strOut += change;
        }

        // Check did string is finish. If not call function again whit new start index
        if (nextIndex >= 0 && nextIndex < strIn.length)
            QnLocale._format(strIn, listArg, noEsc, result, nextIndex);
    } else {
        result.strOut += strIn;
    }
};


// Check if we've loaded jQuery and extend it
if (typeof jQuery !== 'undefined') {

    jQuery.fn.toStrEx = function () {
        var str = "";
        var comp = this;
        if (comp.length > 1) {
            str = "count=" + comp.length;
            comp = comp.get(0);
            str += " ";
        }
        str += comp.prop("tagName");
        if (comp.attr("id"))
            str += " id=" + comp.attr("id");
        if (comp.attr("name"))
            str += " name=" + comp.attr("name");
        if (comp.attr("class"))
            str += " class=" + comp.attr("class");
        return str;
    };

    /**
     * Set text to this component loading string by {@link #getStr} with given string UID.
     * It can use text() or html() depending on the format of the entry.
     * Must be called ONLY with single component!
     *
     * @param {string} uidStr ID of the string. Must be valid
     * @param {Object} [opts] optional options. See below.
     * @param {Object} [opts.listArg] list with arguments. Optional. See {@link #formatMessage} for format.
     * @param {boolean} [opts.noEsc=false] to NOT escape passed arguments (by default they are escaped)
     * @param {boolean} [opts.isSpec=false] true to try to find first string with UID "[uidStr]_[OS]_[SCHEME]".
     * @param {boolean} [opts.isOpt=false] to NOT escape passed arguments (by default they are escaped)
     * @returns {jQuery} always valid
     */
    jQuery.fn.setTextById = function (uidStr, opts) {
        var len = this.length;
        if (len !== 1)
            throw new Error("Not 1 component but " + len + " for string " + uidStr);
        //log("> Set text '" + uidStr + "' to: " + this.toStrEx());

        // noinspection JSAccessibilityCheck
        var entry = QnLocale._getEntryInt(uidStr, opts);
        // noinspection JSAccessibilityCheck
        var str = QnLocale._formatEntry(uidStr, entry, opts);
        if (!str)
            return this; // If optional
        var isHtml = entry && entry["format"] === QnLocale.FORMAT_HTML;
        if (isLogDbg())
            logDbg("< Set text '" + uidStr + "' to: " + this.toStrEx());
        return this.each(function () {
            var $comp = $(this);
            if (isHtml)
                $comp.html(str);
            else
                $comp.text(str);
        });
    };

    /**
     * Set text to this component loading string by {@link #getStr} with given string UID.
     *
     * @param {string} uidStr ID of the string. Must be valid
     * @param {Object} [opts] optional options. See below.
     * @param {Object} [opts.listArg] list with arguments. Optional. See {@link #formatMessage} for format.
     * @param {boolean} [opts.noEsc=false] to NOT escape passed arguments (by default they are escaped)
     * @param {boolean} [opts.isSpec=false] true to try to find first string with UID "[uidStr]_[OS]_[SCHEME]".
     * @param {boolean} [opts.isOpt=false] to NOT escape passed arguments (by default they are escaped)
     * @returns {jQuery} always valid
     */
    jQuery.fn.setTooltip = function (uidStr, opts) {
        // noinspection JSAccessibilityCheck
        var entry = QnLocale._getEntryInt(uidStr, opts);
        // noinspection JSAccessibilityCheck
        var str = QnLocale._formatEntry(uidStr, entry, opts);
        if (!str)
            return this; // If optional
        return this.each(function () {
            $(this).attr("placeholder", str);
        });
    };

} else {
    logWarn("jQuery NOT loaded!");
}

// Use the Mustache.js ( and also 'Underscore.js') method for escaping HTML strings.
// It's 95% faster compared to jQuery's way.
String.prototype.escapeHtml = (function () {
    var __entityMap = {
        "&": "&amp;",
        "<": "&lt;",
        ">": "&gt;",
        '"': '&quot;',
        "'": '&#39;',
        "/": '&#x2F;'
    };

    return function () {
        return this.replace(/[&<>"'\/]/g, function (s) {
            return __entityMap[s];
        });
    }
})();
